@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE MultiWayIf #-}
 {-# OPTIONS_GHC -fsimpl-tick-factor=150 #-}
@@ -72,15 +73,16 @@ import Control.Applicative
 import Control.Concurrent hiding (ThreadId)
 import qualified Data.Binary.Put as P
 import qualified Data.ByteString as B
+import qualified Data.ByteString.Char8 as B8
 import qualified Data.ByteString.Lazy as BL
+import Data.Char (isPrint)
 import Data.IntMap (IntMap)
 import qualified Data.IntMap as IM
-import Data.Foldable (foldMap)
 import Data.Function hiding (id)
 import Data.List
-import Data.Monoid ((<>), mempty)
 import Data.String (IsString)
 import qualified Data.Text as T
+import qualified Data.Text.Encoding as TE
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.Builder as TB
 import qualified Data.Text.Lazy.Builder.Int as TB
@@ -94,6 +96,15 @@ import GHC.RTS.EventTypes
 import GHC.RTS.Events.Binary
 import GHC.RTS.Events.Incremental
 
+#if !MIN_VERSION_base(4, 8, 0)
+import Data.Foldable (foldMap)
+import Data.Monoid (mempty)
+#endif
+
+#if !MIN_VERSION_base(4, 11, 0)
+import Data.Monoid ((<>))
+#endif
+
 -- | Read an entire eventlog file. It returns an error message if it
 -- encouters an error while decoding.
 --
@@ -257,7 +268,7 @@ buildEventInfo spec' =
           <> " on cap " <> TB.decimal otherCap
         ThreadLabel thread label ->
           "thread " <> TB.decimal thread
-          <> " has label \"" <> TB.fromString label <> "\""
+          <> " has label \"" <> TB.fromText label <> "\""
         RequestSeqGC ->
           "requesting sequential GC"
         RequestParGC ->
@@ -309,11 +320,11 @@ buildEventInfo spec' =
         CapEnable{cap} ->
           "enabled cap " <> TB.decimal cap
         Message msg ->
-          TB.fromString msg
+          TB.fromText msg
         UserMessage msg ->
-          TB.fromString msg
+          TB.fromText msg
         UserMarker markername ->
-          "marker: " <> TB.fromString markername
+          "marker: " <> TB.fromText markername
         CapsetCreate cs ct ->
           "created capset " <> TB.decimal cs
           <> " of type " <> TB.fromString (show ct)
@@ -333,7 +344,7 @@ buildEventInfo spec' =
           <> TB.decimal nsec <> "ns (unix epoch)"
         RtsIdentifier cs i ->
           "capset " <> TB.decimal cs
-          <> ": RTS version \"" <> TB.fromString i <> "\""
+          <> ": RTS version \"" <> TB.fromText i <> "\""
         ProgramArgs cs args ->
           "capset " <> TB.decimal cs
           <> ": args: " <> TB.fromString (show args)
@@ -423,7 +434,7 @@ buildEventInfo spec' =
           "About to call the program entry point"
         PerfName{perfNum, name} ->
           "perf event " <> TB.decimal perfNum
-          <> " named \"" <> TB.fromString name <> "\""
+          <> " named \"" <> TB.fromText name <> "\""
         PerfCounter{perfNum, tid, period} ->
           "perf event counter " <> TB.decimal perfNum
           <> " incremented by " <> TB.decimal (period + 1)
@@ -453,15 +464,45 @@ buildEventInfo spec' =
           <> if isCaf heapProfFlags then " CAF" else ""
         HeapProfSampleBegin {..} ->
           "start heap prof sample " <> TB.decimal heapProfSampleEra
+        HeapProfSampleEnd {..} ->
+          "end prof sample " <> TB.decimal heapProfSampleEra
+        HeapBioProfSampleBegin {..} ->
+          "start heap prof sample " <> TB.decimal heapProfSampleEra
+            <> " at time " <> TB.decimal heapProfSampleTime
+
+
         HeapProfSampleCostCentre {..} ->
           "heap prof sample " <> TB.decimal heapProfId
           <> ", residency " <> TB.decimal heapProfResidency
           <> ", cost centre stack " <> buildCostCentreStack heapProfStack
+
         HeapProfSampleString {..} ->
           "heap prof sample " <> TB.decimal heapProfId
           <> ", residency " <> TB.decimal heapProfResidency
           <> ", label " <> TB.fromText heapProfLabel
 
+        ProfSampleCostCentre {..} ->
+          "cap no " <> TB.decimal profCapset
+          <> ", prof sample " <> TB.decimal profTicks
+          <> ", cost centre stack " <> buildCostCentreStack profCcsStack
+
+        ProfBegin {..} ->
+          "start time profiling, tick interval " <> TB.decimal profTickInterval <> " (ns)"
+
+        UserBinaryMessage {..} ->
+          "binary message " <> TB.fromText (replaceUnprintableWith '.' payload)
+
+-- | Replace unprintable bytes in the message with the replacement character
+replaceUnprintableWith
+  :: Char -- ^ Replacement character
+  -> B.ByteString -- ^ Binary message which may contain unprintable bytes
+  -> T.Text
+replaceUnprintableWith replacement = TE.decodeUtf8 . B8.map replace
+  where
+    replace c
+      | isPrint c = c
+      | otherwise = replacement
+
 buildFilters :: [T.Text] -> Maybe TB.Builder
 buildFilters = foldr g Nothing
   where
@@ -530,7 +571,7 @@ ppEventType = TL.unpack . TB.toLazyText . buildEventType
 buildEventType :: EventType -> TB.Builder
 buildEventType (EventType num dsc msz) =
   TB.decimal num <> ": "
-  <> TB.fromString dsc <> " (size "
+  <> TB.fromText dsc <> " (size "
   <> maybe "variable" TB.decimal msz <> ")"
 
 -- | Pretty prints an 'Event', with clean handling for 'UnknownEvent'
@@ -544,7 +585,7 @@ buildEvent imap Event {..} =
   <> maybe "" (\c -> "cap " <> TB.decimal c <> ": ") evCap
   <> case evSpec of
     UnknownEvent{ ref=ref } ->
-      maybe "" (TB.fromString . desc) $ IM.lookup (fromIntegral ref) imap
+      maybe "" (TB.fromText . desc) $ IM.lookup (fromIntegral ref) imap
     _ -> buildEventInfo evSpec
 
 buildEvent' :: Event -> TB.Builder
