@@ -14,10 +14,15 @@ files = map ("test/"++)
     , "sleep.hy.eventlog"
     , "stop.hT.eventlog"
     , "hello-ghc-8.2.2.eventlog", "hello-ghc-8.6.5.eventlog"
+    , "biographical-samples.eventlog"
+    , "time-prof.eventlog"
+    , "trace-binary-event.eventlog"
+    , "unicode.eventlog"
     ]
 
 
 -- Code to help print the differences between a working test and a failing test.
+diffLines :: String -> String -> String
 diffLines o n = diff 1 (lines o) (lines n)
 
 diff :: Int -> [String] -> [String] -> String
